/*******************************************************************************
 * WRW - WorldRadio Widget
 * by Alexandre Bazyl <wrw@bazyl.com.br>
 *
 * -- version 1.3   :: JUN/2008
 * -- version 1.1.4 :: OCT/2007
 * -- version 1.1.3 :: OCT/2007
 * -- version 1.1.2 :: SEP/2007
 * -- version 1.0b  :: AUG/2007
 *
 * wrcountry.js - funcoes relacionadas ao objeto Country
 *				  Obs: os objetos Country/Radio serao utilizados apenas para
 *				       manter os metodos. A relacao de paises/radios sera
 *					   mantida em um vetor (tipo "struct")
 *
 *******************************************************************************/


/*********************************** PAISES ************************************/
// Vetor de paises
var paises = new Array;

// Definicao dos metodos do objeto Country
Country = new Object();
Country.sort = country_sort;
Country.search = country_search;
Country.add = country_insert;
Country.del = country_delete;
Country.getNewIndex = country_getNewIndex;
Country.populate = country_populate;
Country.save = country_save;
Country.load = country_load;


/*******************************************************************************
 * country_sort - Ordena a estrutura 'paises'. Usado na migracao da
 *				  configuracao antiga.
 *******************************************************************************/
function country_sort()
{
	for (var j=1; j<=paises[0].num-1; j++)
		for (var i=j+1; i<=paises[0].num; i++)
			if(paises[j].nome > paises[i].nome) {
				var temp = { nome:paises[j].nome, num:paises[j].num };
				paises[j].nome = paises[i].nome;
				paises[j].num = paises[i].num;
				paises[i].nome = temp.nome;
				paises[i].num = temp.num;
			}
}


/*******************************************************************************
 * country_search - Busca um pais pelo nome
 * Entrada - nome_pais: nome do pais a ser pesquisado
 * Saida - retorna o numero do pais caso ele seja encontrado ou -1 caso contrario
 *******************************************************************************/
function country_search(nome_pais)
{
	if(paises[0].num==0)
		return(-1);
		
	var lim_inf = 1;
	var lim_sup = paises[0].num;
	var match = false;
	
	if (nome_pais >= paises[lim_inf].nome && nome_pais <= paises[lim_sup].nome) {
		while((lim_inf<=lim_sup) && !match) {
			var pos = Math.ceil((Math.abs(lim_inf) + Math.abs(lim_sup)) / 2);
			if (nome_pais < paises[pos].nome)
				lim_sup = pos - 1;
			else
				if (nome_pais > paises[pos].nome)
					lim_inf = pos + 1;
				else
					match=true;
		}
	}
	
	if (match)
		return(paises[pos].num);
	return(-1);
}


/*******************************************************************************
 * country_insert - Insere um pais na struct paises
 * Entrada - nome: nome do pais a ser incluido
 *            opc: se -1 indica uma nova insercao, caso contrario o pais serah
 *                 inserido com o numero indicado por opc (atualizacao)
 * Saida - retorna o numero do pais incluido
 *******************************************************************************/
function country_insert(nome, opc)
{
	if (opc==-1)
		var num = Country.getNewIndex();
	else var num=opc;
	var pos = 1;
	
	if (paises[0].num > 0)
		while((nome > paises[pos].nome) && (pos <= paises[0].num)) {
			pos++;
			if (pos > paises[0].num) break;
		}

	if (pos<=paises[0].num)	
		for (var i=paises[0].num; i>=pos; i--)
			paises[i+1] = { num:paises[i].num, nome:paises[i].nome };
	
	paises[pos] = { num:num, nome:nome };
	paises[0].num++;
	return(num);
}


/*******************************************************************************
 * country_delete - Exclui um pais da struct paises
 * Entrada - numpais: numero do pais a ser exluido
 *******************************************************************************/
function country_delete(numpais)
{
	// Remove da struct paises
	for(var i=1; i<=paises[0].num; i++) {
		if (paises[i].num == numpais) {
			for (var j=i; j<=paises[0].num-1; j++)
				paises[j]=paises[j+1];
			break;
		}
	}
	paises[paises[0].num] = { num:0, nome:"" };
	paises[0].num--;
}


/*******************************************************************************
 * country_getNewIndex - retorna um numero disponivel para pais
 *******************************************************************************/
function country_getNewIndex()
{
	// Define o contador para paises
	var maior=0;
	for (var i=1; i<=paises[0].num; i++)
		if (paises[i].num >= maior) 
			maior = Math.abs(paises[i].num);
	return(Math.abs(maior+1));
}


/*******************************************************************************
 * country_populate - preenche o popup de paises com os paises existentes 
 *                    na struct 'paises'
 * Entrada - popup: indica qual a popup a ser populada 
 *                  ('popup_plCountry' ou 'popup_backCountry')
 *          numchk: indica o numero do pais que deve ter o valor 'selected'.
 *                  Se -1 entao o 'selected' serah o primeiro item da lista.
 *******************************************************************************/
function country_populate(popup, numchk)
{
	var menu = document.getElementById(popup);
	var first = true;

	// Limpa o popup de paises
	Controller.clearPopup(popup);
	
	for (var i=1; i<=paises[0].num; i++) {
		if (numchk == -1) {
			if (first) {
				var newcountry = new Option(paises[i].nome, paises[i].num, 0, "selected");
				first=false;
			}
			else newcountry = new Option(paises[i].nome, paises[i].num, 0);
		}
		else {
			if (numchk == paises[i].num)
				var newcountry = new Option(paises[i].nome, paises[i].num, 0, "selected");
			else
				var newcountry = new Option(paises[i].nome, paises[i].num, 0);
		}
		menu.options[menu.length] = newcountry;
	}
	// retorna o numero do pais selecionado
	if (paises[0].num > 0) {
		menu.disabled = false;
		return (menu.options[menu.selectedIndex].value);
	}
	return(0);
}


/*******************************************************************************
 * country_save - salva a relacao de paises nas preferencias
 *******************************************************************************/
function country_save() {
	// Salva Paises
	var strpaises = "";
	for (var i=1; i<=Math.abs(paises[0].num); i++)
		strpaises = strpaises + paises[i].num + ";" + paises[i].nome + ";"
	widget.setPreferenceForKey(strpaises, "paises");
}


/*******************************************************************************
 * country_load - carrega a relacao de paises
 *******************************************************************************/
function country_load() {
	// Zera o contador
	paises[0] = { num:0, nome:"" };

	// Carrega a string identificadora
	if(window.widget)
		var strpaises = widget.preferenceForKey("paises");
	else return;

	// Parse dos paises
	i=1;
	if (strpaises > "") {
		var num, nome;	
		while (strpaises.length > 3) {
			num = strpaises.substring(0, strpaises.indexOf(';'));
			strpaises = strpaises.substring(strpaises.indexOf(';')+1);
		
			nome = strpaises.substring(0, strpaises.indexOf(';'));
			strpaises = strpaises.substring(strpaises.indexOf(';')+1);
			paises[i] = { num:num, nome:nome };
			i++;
		}
	}
	paises[0].num = i-1;
	// Ordena os paises
	if (Config.cfgVersionConfig == "0.0.0")		// a partir do widget 1.1, jah eh salvo ordenado
		Country.sort();
}
