/*******************************************************************************
 * WRW - WorldRadio Widget
 * by Alexandre Bazyl <wrw@bazyl.com.br>
 *
 * -- version 1.3   :: JUN/2008
 * -- version 1.1.4 :: OCT/2007
 * -- version 1.1.3 :: OCT/2007
 * -- version 1.1.2 :: SEP/2007
 * -- version 1.0b  :: AUG/2007
 *
 * wrradio.js - funcoes relacionadas ao objeto Radio
 *			    Obs: os objetos Country/Radio serao utilizados apenas para
 *				     manter os metodos. A relacao de paises/radios sera
 *				     mantida em um vetor (tipo "struct")
 *
 *******************************************************************************/


/*********************************** RADIOS ************************************/
// Vetor de radios
var radios = new Array;
// Definicao dos metodos do objeto Radio
Radio = new Object();
Radio.getAttrib = radio_getAttrib;
Radio.sort = radio_sort;
Radio.search = radio_search;
Radio.add = radio_insert;
Radio.del = radio_delete;
Radio.getNewIndex = radio_getNewIndex;
Radio.populate = radio_populate;
Radio.getCodec = radio_getCodec;
Radio.save = radio_save;
Radio.load = radio_load;
Radio.update = radio_newUpdate;		///////

//Current = new Object();
//Current.radio = 0;
//Current.country = 0;


/*******************************************************************************
 * radio_getAttrib - retorna um determinado atributo de uma determinada radio
 * Entrada:  numradio - codigo da radio a ser procurada
 *          attributo - o atributo a ser procurado 
 *                      ("nome", "link", "pais" ou "codec")
 * Saida: retorna o valor do atributo desejado
 *******************************************************************************/
function radio_getAttrib(numradio, attributo) 
{
	for (var i=1; i<=radios[0].num; i++) {
		if (radios[i].num == numradio) {
			if (attributo == "nome") return(radios[i].nome);
			else if (attributo == "link") return (radios[i].link);
			else if (attributo == "pais") return (radios[i].pais);
			else if (attributo == "codec") return (Radio.getCodec(radios[i].link));
			else if (attributo == "nome_pais") {
				for (var j=1; j<=paises[0].num; j++)
					if (paises[j].num == radios[i].pais)
						return(paises[j].nome);
			}
			return(null);
		}
	}
}


/*******************************************************************************
 * radio_sort - Ordena a estrutura 'radios'. Usado na migracao da
 *			    configuracao antiga.
 *******************************************************************************/
function radio_sort()
{
	for (var j=1; j<=radios[0].num-1; j++)
		for (var i=j+1; i<=radios[0].num; i++)
			if(radios[j].nome > radios[i].nome) {
				var temp = { num:radios[j].num, nome:radios[j].nome, link:radios[j].link, 
							 pais:radios[j].pais };
				
				radios[j] = { num:radios[i].num, nome:radios[i].nome, link:radios[i].link,
							  pais:radios[i].pais };
				
				radios[i] = { num:temp.num, nome:temp.nome, link:temp.link,
							  pais:temp.pais };
			}
}


/*******************************************************************************
 * radio_search - Busca uma radio pelo nome
 * Entrada - nome_radio: nome da radio a ser pesquisada
 * Saida - retorna o numero da radio caso seja encontrada ou -1 caso contrario
 *******************************************************************************/
function radio_search(nome_radio)
{
	if (radios[0].num == 0 || !radios[0].num)
		return(-1);

	var lim_inf = 1;
	var lim_sup = Math.abs(radios[0].num);
	var match = false;
	
	if (nome_radio >= radios[lim_inf].nome && nome_radio <= radios[lim_sup].nome) {
		while((lim_inf<=lim_sup) && !match) {
			var temp_pos = (Math.abs(lim_inf) + Math.abs(lim_sup) ) / 2;
			var pos = Math.ceil(temp_pos);
			//var pos = Math.ceil((lim_inf + lim_sup) / 2);  // Math.ceil
			if (nome_radio < radios[pos].nome)
				lim_sup = pos - 1;
			else
				if (nome_radio > radios[pos].nome)
					lim_inf = pos + 1;
				else
					match=true;
		}
	}
	
	if (match)
		return(radios[pos].num);
	return(-1);
}


/*******************************************************************************
 * radio_insert - Insere uma radio na struct radios
 * Entrada - nome: nome da radio a ser incluida
 *           url, pais: dados da radio a ser incluida
 *           opc: se -1 indica uma nova insercao, caso contrario a radio serah
 *                inserida com o numero indicado por opc (atualizacao)
 * Saida - retorna o numero da radio incluida
 *******************************************************************************/
function radio_insert(nome, url, pais, opc)
{
	if (opc==-1) 
		var num = Radio.getNewIndex();
	else var num=opc;
	var pos = 1;
	
	if(radios[0].num > 0)
		while((nome > radios[pos].nome) && (pos <= radios[0].num)) {
			pos++;
			if (pos > radios[0].num) break;
		}
	
	if (pos<=radios[0].num)	
		for (var i=Math.abs(radios[0].num); i>=pos; i--)
			radios[i+1] = { num:radios[i].num, nome:radios[i].nome, link:radios[i].link, 
							pais:radios[i].pais };
	
	radios[pos] = { num:num, nome:nome, link:url, pais:pais };
	radios[0].num++;
	return(num);
}


/*******************************************************************************
 * radio_delete - Exclui uma radio da struct radios
 * Entrada - numradio: numero da radio a ser exluida
 *******************************************************************************/
function radio_delete(numradio)
{
	// Remove da struct radios
	for(var i=1; i<=radios[0].num; i++) {
		if (radios[i].num == numradio) {
			for (var j=i; j<=radios[0].num-1; j++)
				radios[j]=radios[j+1];
			break;
		}
	}
	radios[radios[0].num] = { num:0, nome:"", link:"", pais:0 };
	radios[0].num--;
}


/*******************************************************************************
 * radio_getNewIndex - retorna um numero disponivel para um nova radio
 *******************************************************************************/
function radio_getNewIndex()
{
	// Define o contador para radios
	var maior=0;
	for (var i=1; i<=radios[0].num; i++)
		if (radios[i].num >= maior)
			maior = Math.abs(radios[i].num);
	return(Math.abs(maior+1));
}


/*******************************************************************************
 * radio_populate - preenche a popup de radios com as radios pertencentes a 
 *                  um determinado pais.
 * Entrada -      popup: indica a popup a ser populada
 *                       ('popup_plRadio' ou 'popup_backRadio')
 *              numpais: indica o numero do pais selecionado
 *         mem_numradio: se a origem foi o pressionamento de um botao de
 *                       memoria entao contem o numero da radio e -1 caso
 *                       contrario. Indicara qual item serah 'selected'.
 *******************************************************************************/
function radio_populate(popup, numpais, mem_numradio) 
{
	// Se a lista de paises estiver vazia, retorna
	if(numpais==0) return;
	
	var radio_popup = document.getElementById(popup);
	var newradio;
	var first = true;
	
	// Limpa a popup de radios
	Controller.clearPopup(popup);
	
	for (var i=1; i<=radios[0].num; i++)
		if (radios[i].pais == numpais) {
			if(radio_popup.disabled)
				radio_popup.disabled = false;
			// Nao foi pelo pressionamento de um botao de memoria
			if (mem_numradio == -1) {
				if(first) {
					var newradio = new Option(radios[i].nome, radios[i].num, 0, "selected");
					first=false;
				}
				else newradio = new Option(radios[i].nome, radios[i].num, 0);
			}
			else {	// Existe um botao de memoria pressionado
				if (mem_numradio == radios[i].num)
					var newradio = new Option(radios[i].nome, radios[i].num, 0, "selected");
				else
					var newradio = new Option(radios[i].nome, radios[i].num, 0);
			}
			radio_popup.options[radio_popup.length] = newradio;
		}
}


/*******************************************************************************
 * radio_getCodec - verifica a extensao da streaming e retorna o tipo de codec
 *                  que deve ser utilizado
 * Entrada: url: string que contem o link para um determinada radio
 *******************************************************************************/
function radio_getCodec(url) {
	var tam=url.length;
	var ext=url.substring(tam-3,tam);
	ext=ext.toLowerCase();
	var ext5=url.substring(tam-5,tam);
	ext5=ext5.toLowerCase();
	
	if (ext=="pls" || ext=="mp3" || ext=="m3u")
		return("mp3");
	if (ext=="ram" || ext==".rm" || ext5==".smil")
		return("real");
	return("wmp");
}


/*******************************************************************************
 * radio_save - Salva as radios nas preferencias
 *******************************************************************************/
function radio_save() {
	// Salva radios
//	alert("Inicio save radios");
	var strradios = "0";
	var strradios_categoria = "radios";
	if (radios[0].num > 0) {
		var strtotal = "" + radios[0].num;
		widget.setPreferenceForKey(strtotal, "radios");
//	alert("SAVE="+radios[0].num+" (str="+strtotal+")");
		for (var i=1; i<=Math.abs(radios[0].num); i++) {
			strradios_categoria = "radio" + i;
			strradios = radios[i].num+";" + radios[i].nome+";" + radios[i].link+";" + radios[i].pais+";"
			widget.setPreferenceForKey(strradios, strradios_categoria);
		}
	}
	else widget.setPreferenceForKey(strradios, "radios");
//	alert("Fim save radios");
}


/*******************************************************************************
 * radio_load - carrega as radios
 *******************************************************************************/
function radio_load() {
	// Zera os contadores
	radios[0] = { num:0, nome:"", link:"", pais:"" };

	// Carrega a string identificadora
	if(window.widget)
		var contradios = widget.preferenceForKey("radios");
	else return;
	
	if (!contradios)
		contradios=0;
		
	// Parse das radios
	i=1;
	if (contradios > 0) {
		var strradios, strradios_categoria;
		var rlink, rpais;
		for (i=1; i<=contradios; i++) {
			strradios_categoria = "radio" + i;
			
			strradios = widget.preferenceForKey(strradios_categoria);
			num = strradios.substring(0, strradios.indexOf(';'));
			
			strradios = strradios.substring(strradios.indexOf(';')+1);
			nome = strradios.substring(0, strradios.indexOf(';'));
			
			strradios = strradios.substring(strradios.indexOf(';')+1);
			rlink = strradios.substring(0, strradios.indexOf(';'));
			
			strradios = strradios.substring(strradios.indexOf(';')+1);
			rpais = strradios.substring(0, strradios.indexOf(';'));
									
			radios[i] = { num:num, nome:nome, link:rlink, pais:rpais };
		}
	}
	radios[0].num = contradios;
	// Ordena as radios
	if (Config.cfgVersionConfig == "0.0.0")		// a partir do widget 1.1, jah eh salvo ordenado
		Radio.sort();
		
//	alert("Radios Lida="+contradios);
}


/*******************************************************************************
 * radio_update - Verifica e executa a operacao que deve ser realizada com a 
 *                radio lida do arquivo xlm
 * Entrada: nome,url,num_pais,num_radio - dados da radio
 *          operation - operacao a ser realizada (ADD, DEL ou REN)
 *******************************************************************************/
function radio_newUpdate(nome, url, num_pais, num_radio, operation)
{
	var ope=operation.substring(0,3);

	if(ope=="ADD") {		// Insert/Update(url) radio
		if (num_radio==-1)			// -1 indica que eh uma nova numeracao (nova radio)
			Radio.add(nome, url, num_pais, -1);
		else {
			Radio.del(num_radio);
			Radio.add(nome, url, num_pais, num_radio);
		}
	}
	else if(ope=="DEL") {	// Delete radio
		if(num_radio!=-1) 		// Se a radio existe, entao apaga
			Radio.del(num_radio);
	}
	/* Nao utilizado (pelo menos por enquanto)
	else if(ope=="REN") {	// Rename radio
		if(num_radio!=-1) {
			var novo_nome = operation.substring(operation.indexOf('::')+2);
			Radio.del(num_radio);
			Radio.add(novo_nome, url, num_pais, num_radio);
		}
	}
	*/
}
